﻿package TestProject02
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	public class Main extends Sprite 
	{
		public function Main():void 
		{
			if (stage) OnInit();
			else addEventListener(Event.ADDED_TO_STAGE, OnInit);
		}
		
		private static const BLOCK_SIZEX:uint = 6;
		private static const BLOCK_SIZEY:uint = 6;
		private static const COLORS:Array = [0x404040, 0x8080FF, 0x804000, 0xFFFFFF];
		private static const CALC_EVERY_FRAMES:uint = 3;
		
		private var m_FrameCount:uint;
		private var m_BlockCountX:uint;
		private var m_BlockCountY:uint;
		private var m_BlockCount:uint;
		// Bit 0x1 set means currently alive
		// Bit 0x2 set means previously alive.
		// Higer bits contain older history, not used.
		private var m_BlockValues:Array = new Array();
		
		private function OnInit(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, OnInit);
			addEventListener(Event.ENTER_FRAME, OnEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, OnMouseMove);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, OnMouseDown);
			
			m_BlockCountX = stage.stageWidth  / BLOCK_SIZEX;
			m_BlockCountY = stage.stageHeight / BLOCK_SIZEX;
			m_BlockCount = m_BlockCountX * m_BlockCountY;
			
			var i:uint;
			for (i = 0; i < m_BlockCount; i++)
				m_BlockValues[i] = Math.random() > 0.5 ? 1 : 0;
		}
		
		private function OnEnterFrame(e:Event = null):void
		{
			if (m_FrameCount % CALC_EVERY_FRAMES == 0)
				CalcFrame();
			RenderFrame();
			m_FrameCount++;
		}
		
		private function CalcFrame():void
		{	
			var i:uint, x:uint, y:uint;
			for (i = 0; i < m_BlockCount; i++)
				m_BlockValues[i] <<= 1;

			i = 0;
			for (y = 0; y < m_BlockCountY; y++)
			{
				for (x = 0; x < m_BlockCountX; x++)
				{
					var aliveNeighbourCount:uint = 0;
					// left
					if (m_BlockValues[y * m_BlockCountX + (x + m_BlockCountX - 1) % m_BlockCountX] & 0x2)
						aliveNeighbourCount++;
					// right
					if (m_BlockValues[y * m_BlockCountX + (x + 1) % m_BlockCountX] & 0x2)
						aliveNeighbourCount++;
					// row above
					if (m_BlockValues[(y + m_BlockCountY - 1) % m_BlockCountY * m_BlockCountX + (x + m_BlockCountX - 1) % m_BlockCountX] & 0x2)
						aliveNeighbourCount++;
					if (m_BlockValues[(y + m_BlockCountY - 1) % m_BlockCountY * m_BlockCountX + x] & 0x2)
						aliveNeighbourCount++;
					if (m_BlockValues[(y + m_BlockCountY - 1) % m_BlockCountY * m_BlockCountX + (x + 1) % m_BlockCountX] & 0x2)
						aliveNeighbourCount++;
					// row below
					if (m_BlockValues[(y + 1) % m_BlockCountY * m_BlockCountX + (x + m_BlockCountX - 1) % m_BlockCountX] & 0x2)
						aliveNeighbourCount++;
					if (m_BlockValues[(y + 1) % m_BlockCountY * m_BlockCountX + x] & 0x2)
						aliveNeighbourCount++;
					if (m_BlockValues[(y + 1) % m_BlockCountY * m_BlockCountX + (x + 1) % m_BlockCountX] & 0x2)
						aliveNeighbourCount++;
					
					var oldAlive:uint = (m_BlockValues[i] & 0x2) ? 1 : 0, newAlive:uint = oldAlive;
					if (oldAlive)
					{
						if (aliveNeighbourCount < 2 || aliveNeighbourCount > 3)
							newAlive = 0;
					}
					else
					{
						if (aliveNeighbourCount == 3)
							newAlive = 1;
					}
					m_BlockValues[i] |= newAlive;
					i++;
				}
			}
		}
		
		private function RenderFrame():void
		{	
			graphics.clear();
			var pixelX:uint, pixelY:uint = BLOCK_SIZEY / 2;
			const radius:uint = BLOCK_SIZEX / 2;
			var i:uint = 0, x:uint, y:uint;
			for (y = 0; y < m_BlockCountY; y++)
			{
				pixelX = BLOCK_SIZEX/2;
				for (x = 0; x < m_BlockCountX; x++)
				{
					graphics.beginFill(COLORS[m_BlockValues[i] & 0x3]);
					//graphics.drawRect(pixelX, pixelY, BLOCK_SIZEX, BLOCK_SIZEY);
					graphics.drawCircle(pixelX, pixelY, radius);
					pixelX += BLOCK_SIZEX;
					i++;
				}
				pixelY += BLOCK_SIZEY;
			}
		}
		
		private function PutBlock(stageX:Number, stageY:Number):void
		{
			var x:uint = stageX / BLOCK_SIZEX;
			var y:uint = stageY / BLOCK_SIZEY;
			if (x < m_BlockCountX && y < m_BlockCountY)
				m_BlockValues[y * m_BlockCountX + x] |= 0x1;
		}
		
		private function OnMouseMove(event:MouseEvent):void
		{
			if (event.buttonDown)
				PutBlock(event.stageX, event.stageY);
		}

		private function OnMouseDown(event:MouseEvent):void
		{
			PutBlock(event.stageX, event.stageY);
		}
	}
}